<?php
// $Id$

/**
 * @file
 * Provides the Keys API's administrative interface.
 */

/*******************************************************************************
 * Callback Functions, Forms, and Tables
 ******************************************************************************/

/**
 * Keys API List.
 */
function keys_ui_list() {
  $list = keys_get_services();

  if (!$list) {
    return t("No modules that use this feature have been installed.");
  }

  $keys = keys_get_keys();

  return theme('keys_ui_list', $keys);
}

/**
 * Keys API add key form.
 */
function keys_ui_key_form($form_state, $kid = NULL) {
  if (is_numeric($kid)) {
    $result = db_query("SELECT kid, service, rule, api_key FROM {keys_api} WHERE kid = %d", $kid);
    $key_data = db_fetch_object($result);
  } else {
    $key_data = new stdClass();
    $key_data->kid = '';
    $key_data->service = '';
    $key_data->rule = '';
    $key_data->api_key = '';
  }

  $form['kid'] = array(
    '#type' => 'hidden',
    '#value' => $key_data->kid
  );

  $options = array('select' => t("-select service-"));
  $list = array();
  foreach (keys_get_services() as $module) {
    foreach ($module as $value) {
      $list[ucwords($value['module'])][$value['name']] = $value['name'] .' ('. $value['description'] .')';
    }
  }
  $form['service'] = array(
    '#type' => 'select',
    '#title' => t("Service"),
    '#description' => t("The Module specified Service name which the key is used for."),
    '#required' => TRUE,
    '#maxlength' => 60,
    '#default_value' => $key_data->service,
    '#options' => array_merge($options, $list)
  );
  $form['rule'] = array(
    '#type' => 'textfield',
    '#title' => t("Rule"),
    '#description' => t("The rule the supplied key is registered for. For example: example.com, sub.example.com, localhost, localhost:9000."),
    '#default_value' => ($key_data->rule) ? $key_data->rule : keys_get_url(),
    '#required' => TRUE,
    '#attributes' => (!$key_data->rule) ? array('style' => 'color:#999999;') : array()
  );
  $form['api_key'] = array(
    '#type' => 'textfield',
    '#title' => t("Key"),
    '#description' => t("Add API Key you want to manage."),
    '#default_value' => $key_data->api_key,
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t("Save"),
  );
  $form['#redirect'] = 'admin/settings/keys';

  return $form;
}

function keys_ui_key_form_validate($form, &$form_state) {
  if ($form_state['values']['service'] == 'select') {
    form_set_error('service', t("Please select a service for your key."));
  }
}

function keys_ui_key_form_submit($form, &$form_state) {
  $kid = $form_state['values']['kid'];
  $service = $form_state['values']['service'];
  $rule = ($form_state['values']['rule'] != '') ? $form_state['values']['rule'] : keys_get_url();
  $api_key = $form_state['values']['api_key'];

  $key_data = array(
    'kid' => $kid,
    'service' => $service,
    'rule' => $rule,
    'api_key' => $api_key
  );
  keys_save_key($key_data);

  $message = t('Key successfully saved.');
  drupal_set_message($message);
}

function keys_ui_key_delete_form($form_state, $kid) {
  if (!is_numeric($kid)) {
    drupal_not_found();
    exit();
  }

  $key = keys_get_key($kid, array('service', 'rule'));
  $form['kid'] = array(
    '#type' => 'value',
    '#value' => $kid
  );

  return confirm_form($form,
    t("Are you sure you want to delete key for '%service' @ '%rule'?", array('%service' => $key['service'], '%rule' => $key['rule'])),
    'admin/build/share',
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

function keys_ui_key_delete_form_submit($form, &$form_state) {
  db_query("DELETE FROM {keys_api} WHERE kid = %d", $form_state['values']['kid']);
  drupal_set_message(t('Key deleted successfully.'));
  $form_state['redirect'] = 'admin/settings/keys';
}

/*******************************************************************************
 * Module and Helper Functions
 ******************************************************************************/

function keys_ui_delete_key() {
  if ($_SERVER['REQUEST_METHOD'] == 'POST') {
    db_query("DELETE FROM {keys_api} WHERE rule = '%s' AND service = '%s'", $_POST['name'], $_POST['service']);
    print keys_ui_get_keys();
  }
  exit();
}

/*******************************************************************************
 * Theme Functions
 ******************************************************************************/

function theme_keys_ui_list($keys) {
  if (empty($keys)) {
    return t("No keys have been saved.");
  }

  $headers = array(
    array('data' => t('Service')),
    array('data' => t('Rule')),
    array('data' => t('API Key')),
    array('data' => '')
  );
  $rows = array();

  $links = array(
    'edit' => array('title' => 'Edit'),
    'delete' => array('title' => 'Delete')
  );

  foreach ($keys as $key) {
    $links['edit']['href'] = 'admin/settings/keys/'. $key['kid'] .'/edit';
    $links['delete']['href'] = 'admin/settings/keys/'. $key['kid'] .'/delete';
    $rows[] = array(
      array('data' => $key['service']),
      array('data' => $key['rule']),
      array('data' => $key['api_key']),
      array('data' => theme('links', $links))
    );
  }

  return theme('table', $headers, $rows);
}

function theme_keys_ui_key_form($form) {
  $base_url = keys_get_url();
  drupal_add_js(array('keys' => array('baseUrl' => $base_url)), 'setting');
  drupal_add_js(drupal_get_path('module', 'keys_ui') .'/keys_ui.js');
  return drupal_render($form);
}
